@@ -6,11 +6,17 @@
 
 """Utilities and wrappers around inspect module"""
 
+from __future__ import print_function
+
 import inspect
 import re
 
 # Local imports:
 from spyderlib.utils import encoding
+from spyderlib.py3compat import (is_text_string, builtins, get_meth_func,
+                                 get_meth_class_inst, get_meth_class,
+                                 get_func_defaults, to_text_string)
+
 
 SYMBOLS = r"[^\'\"a-zA-Z0-9_.]"
 
@@ -41,14 +47,16 @@ def getobj(txt, last=False):
             return token
     except IndexError:
         return None
-    
+
+
 def getobjdir(obj):
     """
     For standard objects, will simply return dir(obj)
     In special cases (e.g. WrapITK package), will return only string elements
     of result returned by dir(obj)
     """
-    return [item for item in dir(obj) if isinstance(item, basestring)]
+    return [item for item in dir(obj) if is_text_string(item)]
+
 
 def getdoc(obj):
     """
@@ -74,7 +82,7 @@ def getdoc(obj):
     # yield anything, either. So assume the most commonly used
     # multi-byte file encoding (which also covers ascii). 
     try:
-        docstring = unicode(docstring, 'utf-8')
+        docstring = to_text_string(docstring)
     except:
         pass
     
@@ -91,13 +99,13 @@ def getdoc(obj):
             doc['docstring'] = docstring
             return doc
         if inspect.ismethod(obj):
-            imclass = obj.im_class
-            if obj.im_self is not None:
+            imclass = get_meth_class(obj)
+            if get_meth_class_inst(obj) is not None:
                 doc['note'] = 'Method of %s instance' \
-                              % obj.im_self.__class__.__name__
+                              % get_meth_class_inst(obj).__class__.__name__
             else:
                 doc['note'] = 'Unbound %s method' % imclass.__name__
-            obj = obj.im_func
+            obj = get_meth_func(obj)
         elif hasattr(obj, '__module__'):
             doc['note'] = 'Function of %s module' % obj.__module__
         else:
@@ -112,12 +120,9 @@ def getdoc(obj):
                 doc['name'] = name + ' lambda '
                 doc['argspec'] = doc['argspec'][1:-1] # remove parentheses
         else:
-            # Try to extract the argspec from the first docstring line
-            docstring_lines = doc['docstring'].split("\n")
-            first_line = docstring_lines[0].strip()
-            argspec = getsignaturesfromtext(first_line, '')
+            argspec = getargspecfromtext(doc['docstring'])
             if argspec:
-                doc['argspec'] = argspec[0]
+                doc['argspec'] = argspec
                 # Many scipy and numpy docstrings begin with a function
                 # signature on the first line. This ends up begin redundant
                 # when we are using title and argspec to create the
@@ -127,10 +132,12 @@ def getdoc(obj):
                 # the non-whitespace characters on the first line 
                 # match *exactly* the combined function title 
                 # and argspec we determined above.
-                name_and_argspec = doc['name'] + doc['argspec']
-                if first_line == name_and_argspec:
+                signature = doc['name'] + doc['argspec']
+                docstring_blocks = doc['docstring'].split("\n\n")
+                first_block = docstring_blocks[0].strip()
+                if first_block == signature:
                     doc['docstring'] = doc['docstring'].replace(
-                                              name_and_argspec, '', 1).lstrip()
+                                                     signature, '', 1).lstrip()
             else:
                 doc['argspec'] = '(...)'
         
@@ -140,6 +147,7 @@ def getdoc(obj):
         
     return doc
 
+
 def getsource(obj):
     """Wrapper around inspect.getsource"""
     try:
@@ -155,35 +163,66 @@ def getsource(obj):
     except (TypeError, IOError):
         return
 
-def getsignaturesfromtext(text, objname):
+
+def getsignaturefromtext(text, objname):
     """Get object signatures from text (object documentation)
     Return a list containing a single string in most cases
     Example of multiple signatures: PyQt4 objects"""
-    #FIXME: the following regexp is not working with this example of docstring:
-    # QObject.connect(QObject, SIGNAL(), QObject, SLOT(), Qt.ConnectionType=Qt.AutoConnection) -> bool QObject.connect(QObject, SIGNAL(), callable, Qt.ConnectionType=Qt.AutoConnection) -> bool QObject.connect(QObject, SIGNAL(), SLOT(), Qt.ConnectionType=Qt.AutoConnection) -> bool
     if isinstance(text, dict):
         text = text.get('docstring', '')
-    return re.findall(objname+r'\([^\)]+\)', text)
+    # Regexps
+    oneline_re = objname + r'\([^\)].+?(?<=[\w\]\}\'"])\)(?!,)'
+    multiline_re = objname + r'\([^\)]+(?<=[\w\]\}\'"])\)(?!,)'
+    multiline_end_parenleft_re = r'(%s\([^\)]+(\),\n.+)+(?<=[\w\]\}\'"])\))'
+    # Grabbing signatures
+    sigs_1 = re.findall(oneline_re + '|' + multiline_re, text)
+    sigs_2 = [g[0] for g in re.findall(multiline_end_parenleft_re % objname, text)]
+    all_sigs = sigs_1 + sigs_2
+    # The most relevant signature is usually the first one. There could be
+    # others in doctests but those are not so important
+    if all_sigs:
+        return all_sigs[0]
+    else:
+        return ''
+
+# Fix for Issue 1953
+# TODO: Add more signatures and remove this hack in 2.4
+getsignaturesfromtext = getsignaturefromtext
+
+
+def getargspecfromtext(text):
+    """
+    Try to get the formatted argspec of a callable from the first block of its
+    docstring
+    
+    This will return something like
+    '(foo, bar, k=1)'
+    """
+    blocks = text.split("\n\n")
+    first_block = blocks[0].strip()
+    return getsignaturefromtext(first_block, '')
+
 
 def getargsfromtext(text, objname):
     """Get arguments from text (object documentation)"""
-    signatures = getsignaturesfromtext(text, objname)
-    if signatures:
-        signature = signatures[0]
+    signature = getsignaturefromtext(text, objname)
+    if signature:
         argtxt = signature[signature.find('(')+1:-1]
         return argtxt.split(',')
 
+
 def getargsfromdoc(obj):
     """Get arguments from object doc"""
     if obj.__doc__ is not None:
         return getargsfromtext(obj.__doc__, obj.__name__)
 
+
 def getargs(obj):
     """Get the names and default values of a function's arguments"""
     if inspect.isfunction(obj) or inspect.isbuiltin(obj):
         func_obj = obj
     elif inspect.ismethod(obj):
-        func_obj = obj.im_func
+        func_obj = get_meth_func(obj)
     elif inspect.isclass(obj) and hasattr(obj, '__init__'):
         func_obj = getattr(obj, '__init__')
     else:
@@ -205,7 +244,7 @@ def getargs(obj):
         if isinstance(arg, list):
             args[i_arg] = "(%s)" % ", ".join(arg)
             
-    defaults = func_obj.func_defaults
+    defaults = get_func_defaults(func_obj)
     if defaults is not None:
         for index, default in enumerate(defaults):
             args[index+len(args)-len(defaults)] += '='+repr(default)
@@ -216,6 +255,7 @@ def getargs(obj):
             args.remove('self')
     return args
 
+
 def getargtxt(obj, one_arg_per_line=True):
     """
     Get the names and default values of a function's arguments
@@ -250,8 +290,7 @@ def isdefined(obj, force_import=False, namespace=None):
     base = attr_list.pop(0)
     if len(base) == 0:
         return False
-    import __builtin__
-    if base not in __builtin__.__dict__ and base not in namespace:
+    if base not in builtins.__dict__ and base not in namespace:
         if force_import:
             try:
                 module = __import__(base, globals(), namespace)
@@ -281,16 +320,16 @@ def isdefined(obj, force_import=False, namespace=None):
 
 if __name__ == "__main__":
     class Test(object):
-        def method(self, x, y=2, (u, v, w)=(None, 0, 0)):
+        def method(self, x, y=2):
             pass
-    print getargtxt(Test.__init__)
-    print getargtxt(Test.method)
-    print isdefined('numpy.take', force_import=True)
-    print isdefined('__import__')
-    print isdefined('.keys', force_import=True)
-    print getobj('globals')
-    print getobj('globals().keys')
-    print getobj('+scipy.signal.')
-    print getobj('4.')
-    print getdoc(sorted)
-    print getargtxt(sorted)
+    print(getargtxt(Test.__init__))
+    print(getargtxt(Test.method))
+    print(isdefined('numpy.take', force_import=True))
+    print(isdefined('__import__'))
+    print(isdefined('.keys', force_import=True))
+    print(getobj('globals'))
+    print(getobj('globals().keys'))
+    print(getobj('+scipy.signal.'))
+    print(getobj('4.'))
+    print(getdoc(sorted))
+    print(getargtxt(sorted))
